#!/bin/sh

# e.g CPLD_348_rev15HC.bit

if [ -f /mnt/ramdisk/mc/fcns.shinc ]; then
    source /mnt/ramdisk/mc/fcns.shinc
else
    # use system-installed if newer not available
    source "fcns.shinc"
fi

if [ -f /mnt/ramdisk/utils.sh ]; then
    source /mnt/ramdisk/utils.sh
fi

actComp="/bundle/cpld"
if [ -f /usr/bin/utils.sh ]; then
    source /usr/bin/utils.sh
else
    apiOK() { :; }
    apiNA() { :; }
    apiActive() { :; }
    apiActiveEcho() { :; }
    apiProgress() { :; }
    apiError() { :; }
fi

copyForPfu()
{
    mkdir -p /cfg/pfu/cpld/components
    # get rid of the old files so we don't fill up /cfg
    rm -f /cfg/pfu/cpld/components/*

    # target: update.sh and cpld images
    cp /mnt/ramdisk/cpld/* /cfg/pfu/cpld
    cp /mnt/ramdisk/cpld/components/* /cfg/pfu/cpld/components
}

# =====================================================================
# $1 - full path of the file that we just flashed
# $2 - returns:  variable name to place true/false status
# =====================================================================
verifyDataWritten()
{
    local origFile="$1"
    local isCalcMd5=$(which calcmd5)

    if [ -z "$isCalcMd5" ]; then
        echo "Skipping CPLD data verify due to lack of calcmd5 app"
        eval $2=\""true"\"
        return
    fi

    i2c-codeload | grep -q "There are two types of cplds (as of Nov 2012)."
    if [ $? -eq 0 ]; then
        # skip the verify check if running old i2c-codeload that has a
        # broken --read option
        eval $2=\""true"\"
        return
    fi

    echo "Verifying CPLD data written..."

    local checkFile="/mnt/ramdisk/cpld.check"
    i2c-codeload --target cpld --read "$checkFile"

    local md5Orig=$(calcmd5 $origFile)
    local md5Check=$(calcmd5 $checkFile 0 $(stat -c %s $origFile))

    if [ "$md5Orig" == "$md5Check" ]; then
        eval $2=\""true"\"
    else
        eval $2=\""false"\"
    fi
}

# =====================================================================
# $1 - returns:  variable name to place true/false status
# =====================================================================
checkPrepI2c()
{
    # workaround:  some systems seem to not communicate at first.
    # Run a couple of commands to prime the pump.
    local rc=0
    for i1 in $(seq 0 2); do
        results=$(i2c-codeload --target cpld --currinfo 2>&1)
        rc=$?
        [ "$rc" -eq 0 ] && break

        sleep 0.8
    done

    local isWorking
    [ "$rc" -eq 0 ] && isWorking=true || isWorking=false
    eval "$1=$isWorking"
}

checkPrepI2c i2cIsWorking
if ! $i2cIsWorking; then
    echo "Failure when attempting cpld flash"
    echo "Could not communicate to i2c connection."
    apiError $actComp 1403 "Unrecoverable CPLD flash error."
    exit $LOAD_STATUS_FAILED
fi

listFiles=$(ls components/*.bit 2>/dev/null)
triedFlash=false

# pick the first compatible file found in the folder
apiActive $actComp "Selecting Compatible CPLD File."

local cpldReg="/proc/moose/cpld/post_sc"

for file in $listFiles; do
    i2c-codeload --target cpld --quiet --checkfile "$file"
    if [ $? -eq 0 ]; then

        local fromFileVer=$(strings "$file" | grep "\\rev")
        fromFileVer=${fromFileVer##*\\rev}
        fromFileVer=${fromFileVer%%\\*}; # busybox bug:  can't quote rhs if editing backslash

        if [ "$fromFileVer" == "$(cpldrev)" ]; then
            echo "STATUS: Current CPLD firmware is up-to-date."
            apiActive $actComp "CPLD update not required."
            copyForPfu
            exit $LOAD_STATUS_NOT_ATTEMPTED
        fi

        triedFlash=true
        local baseName="${file##*/}"

        apiActive $actComp "Updating CPLD firmware."
        echo "STATUS: Updating CPLD firmware."

        # We want to flash CPLD.
        # The SC could be in any of various states.  We need to get it to the correct 
        #
        # If we loaded SC, we asked it to Shutdown, codeload, NoReboot, so it should be done loading, and sitting at 0x7F, but shutdown.
        #
        # SC might be in any of these states if it's independent PFU going on:
        #   - 0x7F, either Active, or Shutdown
        #           Action: just in case it's shutdown (while transitioning to/from before/after flashing),
        #                   wait a few seconds for a non-7F.  It it's still 0x7F, then SC is up/active.
        #                   if PRE-CPLD, shutdownCPLD.
        #                   if POST-CPLD, reboot and wait for post boot pause.
        #   - 0x70, flashing
        #           Action: wait for flashing to finish (non 0x70).
        #                   if PRE-CPLD, shutdownCPLD.
        #                   if POST-CPLD, reboot and wait for post boot pause.
        #   - 0x0, or non 70/74, rebooting
        #           Action: if PRE-CPLD, we missed it.  wait for post boot pause.
        #                   if POST-CPLD, wait for post boot pause.
        #
        #   If we wait for the post boot pause, and miss it, we may need to shutdownCPLD to get SC ready.

        local result=0

        # check to see if SC is flashing
        apiActive $actComp "Waiting for SC codeload."
        waitFor "SC_OS_SHUTDOWN_CODELOAD" 1 "SC codeload" result

        if $result; then
            # the SC is loading code, wait for it to finish (up to an hour)
            waitFor "! SC_OS_SHUTDOWN_CODELOAD" $((60*60)) "SC codeload" result
            # since we know it was just loading, then it should be ready for instructions to proceed.
        fi

        # see if it already finished, and perhaps rebooted and stopped at the pause
        waitFor "SC_OS_BOOTING_PAUSED" 1 "CPLD Load Ready" result

        if ! $result; then
            ## stall for a bit to allow SC to get to the desired states
            waitFor "SC_OS_ACTIVE || SC_OS_ACTIVEACTIVE || SC_OS_FAILOVER ||SC_OS_SHUTDOWN_NORMAL || SC_OS_BOOTING_PAUSED" $((20*60)) "CPLD Load Ready" result
            if ! $result; then
                echo "Unable to flash CPLD, SC not in correct state."
                apiError $actComp 1404 "Failure to flash CPLD, invalid SC state."
                exit $LOAD_STATUS_FAILED
            fi
        fi

        waitFor "SC_OS_SHUTDOWN_CODELOAD_CPLD || SC_OS_BOOTING_PAUSED" 1 "CPLD Load Ready" result
        if ! $result; then
            if $preCPLDStyle; then
                # we need to be pre bootup
                /mnt/ramdisk/capi.app --prepare
            else
                # we need to be post bootup
                /mnt/ramdisk/capi.app --rebootsc
            fi
        fi
        waitFor "SC_OS_SHUTDOWN_CODELOAD_CPLD || SC_OS_BOOTING_PAUSED" $((10*60)) "CPLD Load Ready" result
        if ! $result; then
            echo "Unable to flash CPLD, SC not in correct state."
            apiError $actComp 1404 "Failure to flash CPLD, invalid SC state."
            exit $LOAD_STATUS_FAILED
        fi

        for i1 in $(seq 0 2); do
            i2c-codeload --target cpld --flash "$file"

            if [ $? -eq 0 ]; then
                verifyDataWritten "$file" isVerified

                if $isVerified; then
                    copyForPfu
                    apiOK $actComp
                    exit $LOAD_STATUS_SUCCESS
                fi
            fi
            apiActive $actComp "Retrying flashing cpld."
            echo "*Retrying* flashing cpld file $baseName"
        done
    fi
done

if $triedFlash; then
    # if got here, there must have been some sort of unrecoverable problem.
    echo "Failure when attempting cpld flash"
    apiError $actComp 1403 "Unrecoverable CPLD flash error."
    exit $LOAD_STATUS_FAILED
fi

if [ "$listFiles" == "" ]; then
    echo "No cpld files to flash"
    apiNA $actComp
else
    echo "There are cpld files in the bundle, but none are compatible"
    echo "with your device.  Cpld flash not attempted."
    apiError $actComp 1400 "No firmware for this board type in bundle."
fi
exit $LOAD_STATUS_NOT_ATTEMPTED

