#!/bin/sh -e

# == DEFAULTS =========================================================
system_ver=""

# add another ref to this list when you have
# another file to copy
listFileVars="ec ec_atlas expansion fu sc cpld cfg"

# list of xxx/components folders
listCompFolder="ec sc cpld cfg"

workfolder=".temp"
clearMost=false
rmWorkfolder=true

# cache the directory where the running script lives
scriptDir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

kScriptName="${0##*/}"
err()
{
    echo 1>&2 "ERROR [${kScriptName}]: $@"
}

help()
{
    echo -e "
Edit Bundle Tool

Add or overwrite sc, ec, or cpld artifact files into an existing
mc codeload bundle.

Copyright (c) 2012 by Dot Hill Systems Corporation.  All rights reserved.
Written by Ken Bertelson

Usage: $kScriptName 'key=\"a value\"' key2=...

    key             Description of Value
    ---             --------------------
    bundle          Required.  Full path of the bundle to edit or create.

    clearMost       true/false.  If true, will delete all components
                    except those associated with MC
                    (Default: $clearMost)

    cpld            Optional.  Full path of cpld file(s) to add/overwrite
                    to the bundle. Version info is embedded within these files.

    ec              Optional.  Full path of the 2U12/2U24 EC .bin file(s) to
                    add/overwrite to the bundle. Version info is embedded within
                    these files.

    ec_atlas        Optional.  Full path of the Atlas EC .bin file(s) to add/
                    overwrite to the bundle. Version info is embedded within
                    these files.

    fu              Optional.  Full path of the bootloader files to overwrite
                    to the bundle. Version info is embedded within these files.

    expansion       Optional.  Full path of the JBOD and drawer .bin file(s) to
                    add/overwrite to the bundle.

    rmWorkfolder    true/false.  If true, will clean up workfolder files
                    (Default: $rmWorkfolder)

    sc              Optional.  Full path of the SC .fla file to add / overwrite
                    to the bundle.  Version info is embedded within this file.

    system_ver      Optional.  System version string.  Specify if you
                    want to create a release-ready bundle.  Do not specify
                    if you want an MC-only development build.

    workfolder      Optional.  Path of work folder to create.  It can be
                    removed automatically on exit.
                    (Default: $workfolder)

    WORKSON_...     Optional.  You can override one or more values defined
                    in bundleinfo.  key=value format exactly like bundleinfo.

EXAMPLES

    $kScriptName  bundle=P100B020-01.bin sc=mgG100.fla
        # edits existing bundle file to add or overwrite a single sc file
        # while supplying a version string for sc

NOTE

    You can specify multiple files against a single key.  For example, if
    ec needed more than one file, you could do:  'ec=\"file1  file2\"'
"
    exit 0
}

# =====================================================================
# If user entered one or more "WORKSON_...=" variables on the command
# line, override original entries in bundleinfo with these values.
# $1 - Full path name of bundleinfo file
# $2 - Full path name allfiles.md5 to edit since I'm changing bundleinfo
# =====================================================================
overrideWORKSON()
{
    local fileInfo="$1"
    local allFiles="$2"
    local overrides=$(set | grep ^WORKSON_);  # capture all WORKSON_ variables

    if [ ! -f "$fileInfo" ]; then
        err "Could not find file '$fileInfo' containing WORKSON_ values"
        exit 1
    fi

    if [ ! -f "$allFiles" ]; then
        err "Could not find md5 file '$allFiles' when searching for WORKSON_ values"
        exit 1
    fi

    local isEdited=false

    local oldIFS=$IFS
    IFS=$'\n'
        for line in $overrides; do
            # each line is in the format:  key='val1 val2'
            local key="${line%%=*}"

            # comment-out current value if it exists
            sed -i -e "s/^[ \t]*$key/#-$key/ g" \
                "$fileInfo"

            # append this override to the end
            echo "# editbundle.sh Override:" >> "$fileInfo"
            echo "$line" >> "$fileInfo"

            echo "Overridding '$key'"

            isEdited=true
        done
    IFS=$oldIFS

    if $isEdited; then
        # gotta update .md5 file w/ new hash value
        sed -i -e "/\/${fileInfo##*/}/ d" "$allFiles"
        md5sum "$fileInfo" >> "$allFiles"
    fi
}

# =====================================================================
# $1 - full path of the file that we're referencing
# $2 - returns: appropriate INI header for a non-RAIDhead EC
#           i.e. S200B23.bin => DotHill_S
# =====================================================================
generateEcIniHeader()
{
    local fullPath="$1"
    local baseName="${fullPath##*/}"

    # EC header is in format: DotHill_?
    # ? = the identifying char of the version number
    local identifier=`expr "$baseName" : '\([A-Z]\+\).*\.bin'`

    if [ "$identifier" == "" ]; then
        err "$type file invalid. Release identifier couldn't be extracted from filename '$baseName' -- make sure your EC file is named descriptively (i.e. S200B23.bin)."
        exit 1
    fi

    local header="DotHill_$identifier"
    echo "INI header: $header"

    eval $2=\""$header"\"
}

# =====================================================================
# $1 - full path of the file that we're referencing
# $2 - returns: INI header for a CPLD image
#       i.e. 'CPLD_348_33_HC' for voltage = 3.3, type = HC
# =====================================================================
generateCpldIniHeader()
{
    local fullPath="$1"

    local _version
    local _timestamp
    local _type
    local _voltage
    local _platform

    getCpldVersInfo "$fullPath" _version _timestamp _type _voltage _platform

    local header="CPLD_${_platform}_${_voltage//./}_${_type}"
    echo "INI header: $header"
    eval $2=\""$header"\"
}

# =====================================================================
# $1 - full version string
# $2 - returns: version info in the 4-char format
#           i.e. S200B23 => 2023
# =====================================================================
generateEcVersion()
{
    local baseName="$1"

    # EC version needs to be supplied in the 4 character format
    # we can deduce it from the version string
    local majorVersion=`expr "$baseName" : '[A-Za-z]\+\([0-9][0-9]\)'`
    local minorVersion=`expr "$baseName" : '[A-Za-z]\+[0-9]\+[A-Za-z]\([0-9][0-9]\)'`

    if [ "$majorVersion" == "" ] || [ "$minorVersion" == "" ]; then
        err "$type file invalid. Version info couldn't be extracted from version '$baseName' -- make sure your EC file is named descriptively (i.e. S200B23.bin) or contains appropriate version information"
        exit 1
    fi

    local versionString="$majorVersion$minorVersion"
    echo "version string: $versionString"

    eval $2=\""$versionString"\"
}

# =====================================================================
# $1 - full path of the file we're referencing
# $2 - returns: version info in the 4-char format
#           i.e. S200B23.bin => 2023
# =====================================================================
generateEcVersionFromFilename()
{
    local fullPath="$1"
    local baseName="${fullPath##*/}"
    local versionString=`expr "$baseName" : '\(.*\)\.bin'`

    # we can deduce it from the filename
    generateEcVersion "$versionString" $2
}

# =====================================================================
# $1 - full path of the file to examine
# $2 - returns:  variable with "" or Version string found
# $3 - returns:  variable with "" or timestamp found
# $4 - returns:  variable with "" or whobuilt found
# $5 - optional: secondary string to identify.  e.g. "initramfs=" if want
#      to find "@DHSv=initramfs=..."
# =====================================================================
getDHSversInfo()
{
    set +e
        local _fullPath="$1"

        local _keyvalues

        # Assume it is gzipped compressed if matched file extension
        if [ "${_fullPath%.gz}" == "${_fullPath}" ]; then
            _keyvalues=$(strings "$_fullPath" | grep "@DHS")
        else
            _keyvalues=$(zcat "$_fullPath" | strings | grep "@DHS")
        fi

        # if secondary string, only consider those that match it
        if [ -n "$5" ]; then
            _keyvalues=$(echo -e "$_keyvalues" | grep "$5" | sed -e "s/$5// g")
        fi

        local extract=$(echo "$_keyvalues" | grep -m 1 "v=")
        eval $2=\""${extract##*v=}"\"

        extract=$(echo "$_keyvalues" | grep -m 1 "t=")
        eval $3=\""${extract##*t=}"\"

        extract=$(echo "$_keyvalues" | grep -m 1 "w=")
        eval $4=\""${extract##*w=}"\"
    set -e
}

# =====================================================================
# $1 - Full path of the cpld file
# $2 - returns:  variable with "" or Version string found
# $3 - returns:  variable with "" or timestamp found
# $4 - returns:  variable with "" or type of cpld file supports
# $5 - returns:  variable with "" or voltage supported
# $6 - returns:  variable with "" or numeric platform (348 = Gallium)
# =====================================================================
getCpldVersInfo()
{
    set +e
        local cpldFile="$1"

        local extract=$(strings "$cpldFile" | grep "\\rev")
        extract="${extract##*\\rev}"
        extract="${extract%%\\*}"
        # May have "99_rev42" at this point, so get rid of _rev*
        eval $2=\"${extract%%_rev*}\"

        extract=$(strings "$cpldFile" | grep "Date:")
        eval $3=\""${extract##*Date: }"\"

        extract=$(strings "$cpldFile" | grep "Part:")
        extract="${extract##*-7000}"
        eval $4=\""${extract:0:2}"\"

        eval $5=0.0
        strings "$cpldFile" | grep -q '\\33\\'
        if [ $? -eq 0 ]; then
            eval $5=3.3
        fi
        strings "$cpldFile" | grep -q '\\18\\'
        if [ $? -eq 0 ]; then
            eval $5=1.8
        fi

        # pull out the path again - extract the platform
        extract=$(strings "$cpldFile" | grep "Design name:")
        extract="${extract##*60-}"
        extract="${extract%%\\*}"
        eval $6=\""${extract}"\"
    set -e
}

# =====================================================================
# $1 - path of folder that contains both boot_data.bin and boot_meta.bin
# $2 - returns:  version of boot_data.bin
# $3 - returns:  version of boot_meta.bin
# $4 - returns:  true - they match:  they are compatible
#                false - error: file missing or version incompatible
# =====================================================================
areFuBinariesMatching()
{
    local _folderPath="$1"

    local _boot_data="$_folderPath/boot_data.bin"
    local _boot_meta="$_folderPath/boot_meta.bin"

    getDHSversInfo "$_boot_data" _version _build_date _whobuilt "FU0_"

    # match only first string that is at least 8 chars long
    local _metaVer=$(strings -n 8 "$_boot_meta" | head -n 1)
    _metaVer="${_metaVer%$'\r'}";  # remove trailing carriage return on cygwin

    eval $2=\"$_version\"
    eval $3=\"$_metaVer\"

    if [ "$_version" == "$_metaVer" ]; then
        eval $4=true
    else
        eval $4=false
    fi
}

# =====================================================================
# $1 - full path of the file to examine
# $2 - type of the file: ec, expansion, sc, or cpld
# $3 - returns:  variable name to place complete ini file section info
# $4 - components folder that contains this file during constructions
# =====================================================================
getVersInfo()
{
    local fullPath="$1"
    local type=$2
    local componentsFolder="$4"

    local version
    local timestamp

    if [ "$type" == "cpld" ]; then
        local cpldType
        local voltage
        local platform
        getCpldVersInfo "$fullPath" version timestamp cpldType voltage platform

        if [ "$version" == "" ]; then
            err "Cpld .bitstream file invalid.  (Must be created via Universal File Writer)"
            exit 1
        else
            eval $3=\""version=$version
build_date=$timestamp
type=$cpldType
voltage=$voltage
"\"
        fi
        return 0
    fi

    local whobuilt
    getDHSversInfo "$fullPath" version timestamp whobuilt

    if [ "$type" == "fu" ]; then
        local metaVersion
        areFuBinariesMatching "$componentsFolder" version metaVersion theyMatch
        if ! $theyMatch; then
            err "FU binary files are mis-matched or missing:"
            err "Versions: boot_data.bin:$version  boot_meta.bin:$metaVersion"
            exit 1
        fi
        # note that $version was reset by areFuBinariesMatching()
    fi

    # Older EC firmware might not have the appropriate headers
    if [ "$type" == "ec" ] || [ "$type" == "ec_atlas" ] || [ "$type" == "expansion" ]; then
        if [ "$version" == "" ]; then # no header info -- derive version from filename
            generateEcVersionFromFilename "$fullPath" version
        fi
    fi

    if [ "$version" == "" ]; then
        err "$type file invalid.  No embedded version info found."
        exit 1
    fi

    if [ "$timestamp" == "" ]; then
        # unfortunately, sc does not have an embedded timestamp
        timestamp=$(stat -c %y "$fullPath")
        timestamp="${timestamp%%.*}"
    fi

    if [ "$whobuilt" == "" ]; then
        eval $3=\""version=$version
build_date=$timestamp
"\"
    else
        eval $3=\""version=$version
build_date=$timestamp
whobuilt=$whobuilt
"\"
    fi
}

# =====================================================================
# $1 - full path of the file to examine
# $2 - Preface string to contruct for md5sum file entry
# $3 - path of the file containing md5sum's to edit
# $4 - substitute header if basename isn't the desired INI header
# $5 - returns:  variable name to place complete ini file section info
# =====================================================================
getFileProps()
{
    local fullPath="$1"

    local baseName="${fullPath##*/}"
    local baseNameNoDots="${baseName//./_}"

    local headerName # for the version.txt entry
    local fileName # for the md5 edit
    if [ "$4" == "" ]; then # header name not included
        headerName="$baseNameNoDots"
        fileName="$baseName"
    else # header name included
        headerName="$4"
        fileName="$baseName"
    fi

    local preface="$2"

    local md5file="$3"
    local md5sum=$(md5sum "$fullPath")
    echo "${md5sum%% *}  $preface$fileName" >> "$md5file"

    eval $5=\""
[$headerName]
filename=$baseName
size=$(stat -c %s "$fullPath")
md5sum=${md5sum%% *}"\"
}

# =====================================================================
# $1 - Name of the INI section to create
# $2 - full path of the file that we're referencing
# $3 - Version string to assign to this file
# $4 - returns:  variable name to place complete ini file section info
# =====================================================================
getIniVersionRef()
{
    local iniSection="$1"
    local fullPath="$2"
    local version="$3"

    local baseName="${fullPath##*/}"
    local baseNameNoDots="${baseName//./_}"

    eval $4=\""
[$iniSection]
filename_ref=$baseName
version=$version
"\"
}

# =====================================================================
# $1 - tar file to extract
# $2 - working folder to create
# =====================================================================
extract()
{
    local tarFile="$1"
    local tempDir="$2"

    mkdir -p "$tempDir"
    tar -xvf "$tarFile" -C "$tempDir"
}

# =====================================================================
# Wipe out any previous meta data generated for named file.
#
# $1 - type of file e.g. "ec" "sc" etc.
# $2 - Path of the file to reference
# $3 - Path to version.txt to edit
# $4 - OPTIONAL - Path to allfiles.md5 to edit
# =====================================================================
deleteFileRefs()
{
    local type=$1
    local fullPath="$2"
    local pathVersionTxt="$3"
    local pathMd5="$4"

    local baseName="${fullPath##*/}"
    local baseNameNoDots="${baseName//./_}"

    # delete an entire ini [section] (section end denoted by blank line)
    sed -i -e "/\[$baseNameNoDots\]/,/^\$/ d"   "$pathVersionTxt"

    if [ -n "$pathMd5" ]; then
        sed -i -e "/\/$type\/components\/$baseName/ d" "$pathMd5"
    fi
}

# =====================================================================
# Some components require special extra info in the ini file.
# This creates a cross-reference [section] in the ini file.
#
# $1 - type of file e.g. "ec" "sc" etc.
# $2 - Name of the file to reference
# $3 - Path to version.txt to edit
# =====================================================================
addExtraIniSection()
{
    local type=$1
    local file=$2
    local pathVersionTxt="$3"

    local version
    local timestamp
    local whobuilt
    local iniVer
    local refName="?"

    case "$type" in
        "cpld") generateCpldIniHeader "$file" refName;;
        "cpld_summary") refName="cpld_summary";;
        "ec") refName="ec_image_fla";;
        "ec_atlas") refName="ec_atlas_image_fla";;
        "expansion") generateEcIniHeader "$file" refName;;
        "sc") refName="sc_image_fla";;
    esac

    if [ "$refName" != "?" ]; then
        # clean up any previous ref information
        deleteFileRefs $type "$refName" "$pathVersionTxt"

        if [ "$type" == "ec" ] || [ "$type" == "ec_atlas" ] || [ "$type" == "expansion" ] || [ "$type" == "sc" ]; then
            getDHSversInfo "$file" version timestamp whobuilt
        fi

        # massage EC version into a special format for these alt headers
        if [ "$type" == "ec" ] || [ "$type" == "ec_atlas" ] || [ "$type" == "expansion" ]; then
            if [ "$version" == "" ]; then # no header info -- derive version from filename
                generateEcVersionFromFilename "$file" version
            else # version info was supplied in header, but we need to get it into the right format
                generateEcVersion "$version" version
            fi
        fi

        if [ "$type" == "cpld" ] || [ "$type" == "cpld_summary" ]; then
            local cpldType
            local voltage
            local platform
            getCpldVersInfo "$file" version timestamp cpldType voltage platform
        fi

        getIniVersionRef "$refName" "$file" "$version" iniVer
        echo -e "$iniVer" >> "$pathVersionTxt"
    fi
}

# =====================================================================
# $1 - firmware type (ec, ec_atlas, expansion, sc, etc.)
# $2 - returns directory name where the file belongs
# =====================================================================
getComponentDirectory()
{
    local type=$1
    local dirName=$2

    case "$type" in
        "fu") dirName="mc";;
        "ec_atlas") dirName="ec";;
        "expansion") dirName="ec";;
        *) dirName="$type";;
    esac

    eval $2=\""$dirName"\"
}

# =====================================================================
# $1 - string of one or more file names separated by blanks
# $2 - working folder w/ all edited files
# =====================================================================
addOverwriteComponents()
{
    local type=$1
    eval local filesToCopy=\$$1; # indirect reference, e.g. variable: sc

    local typeDir=""
    getComponentDirectory "$type" typeDir

    local compDir="$2/$typeDir/components"
    local iniStr=""
    local needCpldSummary=true

    # You're building a bundle by simply adding files
    # to the appropriate xxx/components folder.  Not all
    # xxx/components have to be there.  The codeload
    # logic will load files if they exist, but won't
    # complain if not.  However, mc is an exception:  you
    # must have mc/components files.

    mkdir -p "$compDir"

    # copy all files, first.  Some components (like FU) have
    # files that depend on each other.  By copying first, we
    # can check that all files match/are compatible.
    for file in $filesToCopy; do
        if [ -n "$file" ]; then
            cp -f "$file" "$compDir"
        fi
    done

    for file in $filesToCopy; do
        if [ -n "$file" ]; then
            # clean up
            deleteFileRefs $typeDir "$file" "$2/system/version.txt" "$2/allfiles.md5"

            # update the ini info
            getFileProps "$file" "./$typeDir/components/" "$2/allfiles.md5" "" iniStr
            echo "$iniStr" >> "$2/system/version.txt"
            iniStr=""
            getVersInfo  "$file" $type iniStr "$compDir"

            echo "$iniStr" >> "$2/system/version.txt"
        fi

        # May need to add an additional section to the ini file...
        if [ "$type" == "cpld" ] && $needCpldSummary; then
            addExtraIniSection "cpld_summary" "$file" "$2/system/version.txt"
            needCpldSummary=false
        fi

        if [ "$type" == "ec" ] || [ "$type" == "ec_atlas" ] || [ "$type" == "expansion" ] || [ "$type" == "sc" ] || [ "$type" == "cpld" ]; then
            addExtraIniSection $type "$file" "$2/system/version.txt"
        fi
    done
}

# =====================================================================
# $1 - version string
# $2 - working folder w/ all edited files
# $3 - output file to write to
# =====================================================================
idAsSystemBundle()
{
    local versionInfo="$1"
    local tempDir="$2"
    local outFile="$3"

    cd "$tempDir"
        echo "" >> "$outFile"
        echo "[system_bundle]" >> "$outFile"
        echo "version=$versionInfo" >> "$outFile"
        echo "build_date=`date`" >> "$outFile"
        echo "" >> "$outFile"
    cd - > /dev/null
}

# =====================================================================
# $1 - Linux-style path
# $2 - Normalized path suited to our operating system (*nix or Windows)
# =====================================================================
normalizePath()
{
    local originalPath="$1"
    local finalPath="$originalPath"

    # If we're running in cygwin, convert to a windows-style path
    if [ -f /usr/bin/cygpath ]; then
        echo "Converting '$originalPath' to Windows-style path..."
        finalPath=$(/usr/bin/cygpath -w "$originalPath" | tr -d "\r\n")
    fi

    echo "Path: $finalPath"
    eval $2=\""$finalPath"\"
}

# =====================================================================
# $1 - System version string; "" if want a development bundle, instead
# $2 - working folder w/ all edited files
# $3 - output .bin file to create/overwrite
# =====================================================================
finalizeBundle()
{
    local versSystem="$1"
    local tempDir="$2"
    local outFile="$3"

    local fullPathOut=$(readlink -f "$outFile")
    if [ -z "$fullPathOut" ]; then
        fullPathOut="$outFile"
    fi

    cd "$tempDir"
        # it's a system bundle
        if [ "$versSystem" != "" ]; then
            idAsSystemBundle "$versSystem" "./"  system/version.txt

            # generate the contents XML manifest
            if [ -f "$scriptDir/makecontents.pl" ]; then
                local makeContentsPath
                local verifyContentsPath
                normalizePath "$scriptDir/makecontents.pl" makeContentsPath
                normalizePath "$scriptDir/verifyContents.pl" verifyContentsPath

                echo -e "Generating contents.xml manifest...\n"
                perl "$makeContentsPath"

                echo -e "Verifying manifest contents...\n"
                perl "$verifyContentsPath" ./contents.xml
                if [ $? -ne 0 ]; then
                    err "Manifest verification FAILED!"
                    err "Contact the MC platform team to fix this issue."
                    exit 1 # Abort the bundle build
                fi

                echo -e "Done with manifest.\n"

                # Remove the scripts inside the bundle
                # (not necessarily the ones we were just running)
                [ -f makecontents.pl   ] && echo "makecontents.pl exists, deleting it."
                rm -f ./makecontents.pl
                [ -f verifyContents.pl ] && echo "verifyContents.pl exists, deleting it."
                rm -f ./verifyContents.pl

                # Also remove them from the MD5 file.
                echo "Cleaning up allfiles.md5..."
                sed -i -e "/.\/makecontents.pl/d"   "./allfiles.md5"
                sed -i -e "/.\/verifyContents.pl/d" "./allfiles.md5"

            fi

            # Before we pack everything up, run md5sum against the current files
            # to make sure nothing is changed or missing.

            echo "Verifying MD5 sums match up..."
            local md5Pass=true

            # This is to "sneak around" the "set -e" up top,
            # since we don't want to stop if md5sum gets a bad
            # return code.
            md5sum -c allfiles.md5 || md5Pass=false

            if [ $md5Pass == false ]
            then
                err "MD5 verification failed. Quitting."
                exit 1
            fi

            echo "Done with MD5 verification."

        fi

        # build-up a comprehensive list of all files
        local manifest=$(find . -type f | sort | sed -e 's~\./~~')

        echo -e "\nCreating output file '$fullPathOut'...\n"

        tar --help 2>&1 | grep -q "numeric\-owner" \
            && permOpts="--numeric-owner --owner=0 --group=0" \
            |  permOpts=""

        tar $permOpts -cvhf \
            "$fullPathOut" $manifest
    cd - > /dev/null

}

# =====================================================================
# MAIN
# =====================================================================
# handle all command-line arguments
[ $# -eq 0 ] && help
while [ $# -gt 0 ]; do
    eval "$1"; shift
done

# =====================================================================
# ARGUMENT VALIDATION
# =====================================================================
if [ ! -f "$bundle" ]; then
    if [ ! -d "$workfolder" ]; then
        err "You must specify an existing bundle=file_that_exists"
        err "-OR- a valid workfolder so that a new bundle may be created."
        exit 1
    fi
fi

for i1 in $listFileVars; do
    eval fileNames=\$$i1
    for file in $fileNames; do
        if [ -n "$file" ]; then
            if [ ! -f "$file" ]; then
                err "$i1 file '$file' could not be found"
                exit 2
            fi
        fi
    done
done

# == CORE LOGIC =======================================================

if [ -f "$bundle" ]; then
    extract  "$bundle"  "$workfolder"
fi

if $clearMost; then
    for i1 in $listCompFolder; do
        rm -rf "$workfolder/$i1/components/"
        mkdir -p "$workfolder/$i1/components/"

        # edit the md5 file to remove all refs
        sed -i -e "/\/$i1\/components\//d" "$workfolder/allfiles.md5"
    done
fi

# maybe build folks wanted to change WORKSON_... variables
cd $workfolder
    overrideWORKSON ./bundleinfo ./allfiles.md5
cd - > /dev/null

# mc has all its version info already, so seed final ini file
cp $workfolder/mc/version.txt  $workfolder/system/version.txt

# copy all the component files specified via cmd line args
for i1 in $listFileVars; do
    addOverwriteComponents  "$i1"   "$workfolder"
done

finalizeBundle  "$system_ver"  "$workfolder"  "$bundle"

if $rmWorkfolder; then
    rm -rf "$workfolder"
fi
