#!/bin/sh

# =====================================================================
# CONSTANTS
MCFU_FAKE_MTDNBR=99
MCFU_MAJOR_FILENBR=200
MCFU_MINOR_FILENBR=1;       # 0==compact flash; 1==FU SPI Flash
# =====================================================================

if [ -f /usr/bin/utils.sh ]; then
    source /usr/bin/utils.sh
else
    apiOK() { :; }
    apiNA() { :; }
    apiActive() { :; }
    apiActiveEcho() { :; }
    apiProgress() { :; }
    apiError() { :; }
fi

msg()
{
    echo "$1"
}

# 1(+) if mounted, 0 if unmounted
mtdMounted()
{
    echo $(mount | grep mtd | wc -l)
    return
}

stopApps()
{
    echo "Disabling soft watchdogs"
    disablewatchdogs
    sleep 3
    echo "Killing watcherd"
    killall -9 watcherd
    echo "Disabling moose watchdog"
    echo "0" > /proc/moose/watchdog/timeout-secs
    shields up;    # kills watchdog timer
    killappsnorpc

    sleep 2 # allow system to clean up
    return
}

unmountAll()
{
    # umount anything with *mtd* in the name
    for PARTITION in `mount | grep mtd | awk {'print $1'}`
    do
        umount $PARTITION
    done
    return
}

unmountApps()
{


    # If the memtracker is using the libpal.so in /app/opt/lib,
    # we need to kill it and bring it back up using the /lib one.
    # Otherwise the following logic will permanently kill the memtracker,
    # and we will lose logging for this codeload.
    lsof -c memtrack | grep /opt/lib/libpal.so
    lsRc=$?

    if [ $lsRc -eq 0 ]
    then 
        cp /mnt/app1/opt/lib/libpal.so /lib/
        pkill memtrack
        memtrack &
        msg "Successfully re-initialized memtrack."
    fi

    # umount anything with *mtd* and *squashfs* in the name
    mount | grep mtd | grep squashfs |
    while read PARTITION on MOUNT_POINT extra
    do
        touch /mnt/ramdisk/do.unmount
        count="5"
        while [ "$count" -gt "0" ];
        do
            umount $PARTITION
            if [ $? -eq 0 ]
            then
                msg "Successfully unmounted $PARTITION filesystem"
                break
            else
                msg "Unable to unmount $PARTITION filesystem, finding processes which are currently using it."
                for PROCESS in `fuser -m $MOUNT_POINT`
                do
                    name=`cat /proc/$PROCESS/cmdline`
                    msg "PID - $PROCESS:$name is using $PARTITION filesystem, mounted on $MOUNT_POINT, need to kill this process...."
                done

                # Killing all processes which are using this filesystem.
                fuser -mk $MOUNT_POINT
                umount $PARTITION
                if [ $? -eq 0 ]
                then
                    msg "Successfully unmounted $PARTITION filesystem, mounted on $MOUNT_POINT, after all running processes killed."
                    break
                fi
            fi
        count=$(($count-1))
        done
    done
    return

}

# 1+ if mounted, 0 if not
cfgMounted()
{
    echo `mount | grep mtd | grep jffs2 | wc -l`
    return
}


startMsgServer()
{
    msg "Starting message server"
    httpdlite.srv start
    

    return
}

# parameter: mtd where CFG is found
migrateCfg()
{
    if [ "$FLASHVER_TGT" -eq "$FLASHVER_CUR" ]
    then
        apiActive $actComp "Flash layoutOK."
        msg "Flash layout indicates match. No migration required."
    fi


    if [ "$FLASHVER_TGT" -eq "5" ]
    then
        apiActive $actComp "Performing flash re-layout."
        msg "Flash layout v.$FLASHVER_CUR, Target v.$FLASHVER_TGT"

        msg "Creating backup of configuration files"
        cd /cfg
        tar -czf /mnt/ramdisk/cfg.tgz *
        cd - > /dev/null
        msg "Backup complete"

        msg "Uncompressing new filesystem"
        cd /mnt/ramdisk/mc/components
        gunzip /mnt/ramdisk/mc/components/cfg.jffs2.gz
        cd - > /dev/null

        umount /mnt/cfg

        if [ -f /mnt/ramdisk/mc/components/cfg.jffs2 ]
        then
            msg "Writing new filesystem to memory"
            loadmtd /mnt/ramdisk/mc/components/cfg.jffs2 \
                    $1          NULL \
                    loadStat

            msg "Re-mounting /dev/mtd$1."
            mount -t jffs2 /dev/mtd$1 /mnt/cfg
            cd /mnt/cfg
            msg "Scrubbing new filesystem"
            rm -rf *
            msg "Restoring previous configuration files"
            tar -xf /mnt/ramdisk/cfg.tgz -C /mnt/cfg
            msg "Recompressing components"
            gzip -9 /mnt/ramdisk/mc/components/cfg.jffs2
        else
            msg "Error: Unable to rebuild new CFG filesystem. Self-healing code will fix this error on next reboot."
        fi
        apiActiveEcho $actComp "Migration complete"
    fi
    return
}

checkForMigration()
{
    if [ "$(migrationRequired)" == "1" ]
    then
        msg "Flash partition migration will be required"
        msg "Current layout v.$FLASHVER_CUR, Target layout .v$FLASHVER_TGT"
        export MIGRATEFLASH=1
    fi
    return
}

# returns 1 if migration is required, 0 if no migration is required
migrationRequired()
{
    # flash layout 3 to <next>
    proposed=$(inicntrl /mnt/ramdisk/mc/version.txt MC_system flash_layout)
    current=$(inicntrl /cfg/version.txt MC_system flash_layout)
    
    if [ -n "$proposed" ] && [ -n "$current" ]
    then
        if [ "$proposed" -gt "$current" ]
        then
            echo "1"
        else
            echo "0"
        fi
        return
    else
        echo "Error: Unable to determine flash layout version. Migration attempting to continue."
    fi

    return
}

#If code was just loaded, keygen will be running
keygenCheck()
{
    local isKeygenRunning=$(pgrep ssh-keygen && echo "true" || echo "false")
    if $isKeygenRunning; then
        killall -9 ssh-keygen
        echo "Waiting for applications to gracefully shutdown"
        sleep 180 
        killall -9 sshd
    fi
}

# copy the top and lower level update.sh scripts over so we can rebuild a bundle in the future. 
copyForPfu()
{
    rm -rf /cfg/pfu/mc/*

    mkdir -p /cfg/pfu/mc/components

#TODO: this is error prone - we really oughta have a manifest for pfu and copy the files listed in the manifest. rd
    cp /mnt/ramdisk/capi.app /cfg/pfu
    cp /mnt/ramdisk/*.sh /cfg/pfu
    cp /mnt/ramdisk/*.xml /cfg/pfu
    cp /mnt/ramdisk/*.awk /cfg/pfu
    cp /mnt/ramdisk/*.bin /cfg/pfu
    cp /mnt/ramdisk/*.cmd /cfg/pfu
    cp /mnt/ramdisk/*.txt /cfg/pfu #targeting mc_boot_meta.txt
    cp /mnt/ramdisk/mc/* /cfg/pfu/mc
    # we can't back up the *.gz anymore, because initramfs.gz is huge
}

# =====================================================================
# *OLD* Mechanism is supported for older systems
# $1 - name of the file to flash
# $2 - Block partition nbr e.g. 5
# $3 - Human description of the partition (for msgs)
# =====================================================================
useFlashcode()
{
    local errPreface="Error writing the Management Controller File"
    local errSuffix="Please retry your request."

    if [ "$1" == "" ] || [ "$2" == "" ] || [ "$3" == "" ]; then
        msg "$errPreface (missing parameter). $errSuffix"
        return
    fi

    msg "Loading the Management Controller $3"

    if [ "$3" == "NULL" ]; then
        flashcode $1 /dev/mtd$2
    else
        flashcode $1 /dev/mtd$2 -c $3
    fi
    msg "Finished loading Management Controller $3"
    return
}

# =====================================================================
# $1 - Block partition nbr e.g. 5
# $2 - returns:  true - treat as readonly; false - it's read/write
# =====================================================================
isMtdReadOnly()
{
    local blkNbr="$1"

    if [ -z "$mtdnbr_kernel" ]; then
        genAllMtdPropVariables "$(cat /proc/moose/mtd)"
    fi

    local allReadOnly="$(cat /proc/moose/mtd-readonly)"
    local myPartNbr

    # remove all blanks and double-quotes
    allReadOnly=${allReadOnly// /_}
    allReadOnly=${allReadOnly//\"/}

    for part in $allReadOnly; do
        eval myPartNbr=\"\$mtdnbr_$part\"
        if [ $myPartNbr -eq $blkNbr ]; then
            eval "$2=true"
            return
        fi
    done

    eval "$2=false"
}

# =====================================================================
# $1 - Block partition nbr e.g. 5
# $2 - Full name to create e.g. "/dev/mtd5Unbuf"
# =====================================================================
mknodMtd()
{
    local partNbr="$1"
    local partName="$2"

    # All work done on temporary unbuffered mtd partition
    rm -f "$partName"

    if [ $partNbr -eq $MCFU_FAKE_MTDNBR ]; then
        mknod "$partName" c $MCFU_MAJOR_FILENBR $MCFU_MINOR_FILENBR;  # mcfu driver
    else
        mknod "$partName" c 90 $((partNbr*2))
    fi
}

# =====================================================================
# *NEW* Mechanism to write to flash partitions
# $1 - name of the file to flash
# $2 - Block partition nbr e.g. 5
# $3 - Human description of the partition (for msgs) or NULL
#
# $4   --src_base          hex or decimal byte offset to extract src
# $5   --dest_base         hex or decimal byte offset to overlay dest
# $6   --src_length        hex or decimal nbr of bytes to copy
#
# $7 - returns:  variable name to place true/false success status
# =====================================================================
useCptomtd()
{
    local fileToFlash="$1"
    local partNbr="$2"
    local humanDesc="${3// /_}";     # remove all blanks

    local rc=false
    local useLogapi=true
    [ "$humanDesc" == "NULL" ] && useLogapi=false

    local strLogapi
    $useLogapi && strLogapi="--logapi $humanDesc"

    local strOffset="--src_base $4 --dest_base $5 --src_length $6"

    local strProtected
    local isReadOnly
    isMtdReadOnly $partNbr isReadOnly
    $isReadOnly && strProtected="--protected"

    msg "Loading the Management Controller $humanDesc"

    # All work done on temporary unbuffered mtd partition
    local unbufPart="/dev/mtd${partNbr}Unbuf"
    mknodMtd $partNbr "$unbufPart"

    # compare input file to flash partition.  Will exit w/ non-zero if different
    $useLogapi && apiNA "$humanDesc" "Comparing Sectors"
    cp-to-mtd --compare $strOffset $strLogapi $fileToFlash $unbufPart 2>&1

    if [ $? -eq 0 ]; then
        rc=true
        $useLogapi && apiNA "$humanDesc" "Flash image is the same, no need to re-flash."
    else
        $useLogapi && apiActive "$humanDesc" "Flash $fileToFlash to /dev/mtd$partNbr"

        # Try to write to flash a few times if we detect problems
        for i1 in $(seq 0 2); do
            cp-to-mtd --write   $strOffset $strLogapi $strProtected $fileToFlash $unbufPart 2>&1
            cp-to-mtd --compare $strOffset $strLogapi --quiet       $fileToFlash $unbufPart 2>&1

            if [ $? -eq 0 ]; then
                $useLogapi && apiOK "$humanDesc"
                msg "Done"
                rc=true
                break
            fi
            local infoMsg="Read Validation Failed.  Rewriting data..."
            msg "$infoMsg"
            $useLogapi && apiActive "$humanDesc" "$infoMsg"
        done
    fi

    if $rc; then
        msg "Finished loading Management Controller $humanDesc"
    else
        msg "Error writing the Management Controller File"
        msg "Could not reliably write to Flash"
        msg " "

        $useLogapi && apiActiveEcho "$humanDesc" "*****************************************************************"
        $useLogapi && apiActiveEcho "$humanDesc" "  Your storage controller did not update correctly.  If it will
        $useLogapi && apiActiveEcho "$humanDesc" "  not boot, please contact your support representative.
        $useLogapi && apiActiveEcho "$humanDesc" "*****************************************************************"
        $useLogapi && apiError "$humanDesc" 1209 "Failure to Flash $humanDesc to /dev/mtd$partNbr"
    fi

    rm -f $unbufPart
    eval "$7=$rc"
}

# =====================================================================
# $1 - StartTime - value returned when calling this fcn w/ no args
# $2 - returns:  seconds elapsed since StartTime
# =====================================================================
timer()
{
    if [ $# -eq 0 ]; then
        echo $(date '+%s')
    else
        local  stime=$1
        etime=$(date '+%s')
        eval $2=$((etime - stime))
    fi
}

# =====================================================================
# $1 - timeout in seconds to wait
# $2 - command string to run
# $3 - returns:  data string (may be empty)
# $4 - returns:  status value as a hex nbr (e.g. "0x00"
# =====================================================================
waitforMcFuCmd()
{
    local timeout="$1"
    local cmd="$2"
    local isComplete=false

    local fuDir="/proc/moose/mcfu"

    local startTime=$(timer)
    local elapsedSecs=0

    local uiWaitTime=5
    local uiTime=$uiWaitTime

    echo "$cmd" > "$fuDir/cmd"

    while [ $elapsedSecs -lt $timeout ]; do

        if [ $uiTime -le $elapsedSecs ]; then
            uiTime=$((elapsedSecs + uiWaitTime))
            echo "Waiting on FU command...($elapsedSecs secs)"
        fi

        sleep 0.3
        local myFuStat=$(cat "$fuDir/cmd")

        if [ "${myFuStat%%,*}" == "done" ]; then
            isComplete=true
            break
        fi
        timer $startTime elapsedSecs
    done

    if $isComplete; then
        eval "$3=\"$(cat $fuDir/data)\""
        eval "$4=${myFuStat#*=}"
    else
        eval $3=
        eval "$4=0xFF"; # complete list of codes in Andy's file: sys_errors.h
    fi
}

# =====================================================================
# Can check FU for "newness" regarding features.  Andy agreed that
# major and minor will always be unique and increment upward.
# (e.g. for 5.78.16314  major=5  and  minor=78)
#
# $1 - returns:  variable name to place true/false success status
# $2 - returns:  variable name to capture version as ulong
# =====================================================================
getFuVersionAsUlong()
{
    local status=true
    local fuData
    local fuStat

    waitforMcFuCmd 10 "get_vol_name 1" fuData fuStat

    if [ $((fustat)) -eq 0 ]; then
        local hi=${fuData##*_}
        hi=${hi%%.*}

        # make Andy's test code always newest
        [[ "${hi#test}" != "$hi" ]] && hi=99

        # normalize "pfu" revision numbers
        [ $hi -gt 100 ] && hi=$(( hi-100 ))

        local mid=${fuData#*.}
        mid=${mid%%.*}

        # Remove any leading zeros.
        mid="$(echo $mid | sed 's/^0*//')"

        eval "$2=$(( (hi*100000) + mid ))"
    else
        status=false
    fi

    eval "$1=$status"
}

# =====================================================================
# $1 - returns:  variable name to place true/false success status
# $2 - returns:  variable name to place Nbr of MB to erase
# =====================================================================
isMcfuWorking()
{
    local status=true
    local devName="/dev/mcfu"

    mknodMtd $MCFU_FAKE_MTDNBR "$devName"

    # Confirm that the drive+FU version appear to be working
    dd if=/dev/mcfu conv=notrunc bs=65536 count=1 &> /dev/null
    if [ $? -ne 0 ]; then
        status=false
    fi

    if $status; then
        local fuVersion=0
        getFuVersionAsUlong status fuVersion

        if $status; then
            # v5.79 is version that has the fixed MC-FU communications

            [ $fuVersion -lt 500079 ] && status=false
        fi
    fi

    if $status; then
        local fuData
        local fuStat

        waitforMcFuCmd 10 "get_msd_info $MCFU_MINOR_FILENBR" fuData fuStat

        if [ $((fustat)) -eq 0 ]; then
            blockSize=${fuData##*blockSize=}
            blockSize=${blockSize%% *}

            numBlocks=${fuData##*numBlocks=}
            numBlocks=${numBlocks%% *}

            totalSize=$(( blockSize*numBlocks ))
            totalMB=$(( totalSize/(1024*1024) ))

            [ $totalMB -le 0 ] && status=false

            eval "$2=$((totalMB/2))"
        else
            status=false
        fi
    fi

    rm -f "$devName"
    eval "$1=$status"
}

# =====================================================================
# The MC and FU bootloaders are special.  They requires that the
# *entire* SPI flash be erased if any images need to be updated.
# $1 - "FU" or "MC"
# $2 - Block partition nbr e.g. 5
# $3 - Human description of the partition (for msgs) or NULL
# $4 - Nbr of MB's of data to erase
# $5 - returns:  was flash erased?  true/false
# $6 - returns:  variable name to place true/false success status
# =====================================================================
maybeEraseSpiFlash()
{
    local fuOrMc=$1
    local partNbr=$2
    local humanDesc="${3// /_}";     # remove all blanks
    local mbToErase=$4
    local flashErased=true
    local status=true

    local devName="/dev/mcfu"
    mknodMtd $partNbr "$devName"

    # compare input files to flash partitions.  Will exit w/ non-zero if different

    # (bl_... values are generated by running bl-metaprops app)
    local src_base="\$bl_${fuOrMc}_vol0_src_base"
        eval "src_base=$src_base"
    local dest_base="\$bl_${fuOrMc}_vol0_dest_base"
        eval "dest_base=$dest_base"
    local src_length="\$bl_${fuOrMc}_vol0_src_length"
        eval "src_length=$src_length"

    apiNA "$humanDesc" "Comparing Sectors"
    cp-to-mtd --compare --src_base $src_base --dest_base $dest_base \
        --src_length $src_length \
        --quiet "${PWD}/components/boot_data.bin" "$devName" 2>&1
    local rc=$?

    if [ $rc -eq 0 ]; then
        src_base="\$bl_${fuOrMc}_vol1_src_base"
            eval "src_base=$src_base"
        dest_base="\$bl_${fuOrMc}_vol1_dest_base"
            eval "dest_base=$dest_base"
        src_length="\$bl_${fuOrMc}_vol1_src_length"
            eval "src_length=$src_length"

        cp-to-mtd --compare --src_base $src_base --dest_base $dest_base \
            --src_length $src_length \
            --quiet "${PWD}/components/boot_data.bin" "$devName" 2>&1
        rc=$?
    fi

    if [ $rc -eq 0 ]; then
        apiNA "$humanDesc" "Flash images are the same, no need to re-flash."
        flashErased=false
    else
        apiActiveEcho "$humanDesc" "Clearing SPI Flash for ${fuOrMc} loader files..."
        tmpFile=all0xFFs

        # create a 1MB 0xFF filled temporary file used to erase FU flash
        tr "\000" "\377" < /dev/zero | \
            dd ibs=1024 count=1024 of=$tmpFile &> /dev/null

        for i1 in $(seq 1 $mbToErase); do
            msg "    Erasing MB $i1 of $mbToErase"
            apiProgress "$humanDesc" $i1 $mbToErase "Erasing MB #$i1"

            # retry the erase a few times if there is a problem
            for i2 in $(seq 0 2); do
                cp-to-mtd --write --dest_base $(( (i1-1)*1024*1024)) \
                    --quiet $tmpFile "$devName" > /dev/null 2>&1
                rc=$?
                [ $rc -eq 0 ] && break
            done
        done
        rm -f $tmpFile
        apiOK "$humanDesc"
    fi

    rm -f "$devName"
    eval "$5=$flashErased"
    eval "$6=$status"
}

# =====================================================================
# $1 - name of the file to flash
# $2 - Block partition nbr e.g. 5
# $3 - Human description of the partition (for msgs)
# $4 - returns:  variable name to place true/false success status
# =====================================================================
loadmtd()
{
    # Use the NEW approach if available

    if [ -n "$(which cp-to-mtd)" ]; then
        # set offset args so that entire file is extracted and copied
        useCptomtd "$1" "$2" "$3" 0 0 -1 "$4"
    else
        useFlashcode "$1" "$2" "$3"
        # (No support of write-protection)
        eval "$4=true";   # no error checking
    fi
}

# =====================================================================
# $1 - string containing contents of mc meta boot (mtd partition)
# $2 - name of variable to return size of zImage
# $3 - name of variable to return size of initramfs
# =====================================================================
getCurImageSizes()
{
    local metaContents="$1"

    oldIFS=$IFS
    IFS=$'\n'
        for line in $metaContents; do
            if [ "${line:0:1}" == $'\xff' ]; then
                break;      # binary data -- we're done
            fi
            if [ "${line:0:1}" == "#" ]; then
                continue;   # skip comments
            fi
            local keyword="${line%%=*}"

            if [ "$keyword" == "bootcmd" ]; then
                # Could be either "pf/" or "sf/", let's just parse for one type
                line=${line//sf/pf}

                # the line to parse looks something like this:
                # bootcmd=boot linux pf/00220000/14c5 pf/00700000/14c5
                local sizeZimage="${line#* pf/}"
                sizeZimage="${sizeZimage#*/}"

                local localsizeInitramfs="${sizeZimage#* pf/}"
                localsizeInitramfs="${localsizeInitramfs#*/}"
                localsizeInitramfs="${localsizeInitramfs%% *}"

                sizeZimage="${sizeZimage%% *}"

                eval $2="$sizeZimage"
                eval $3="$localsizeInitramfs"
            fi
        done
    IFS=$oldIFS
}

# =====================================================================
# Initial implementation of these scripts had a limitation:  each
# component script could communicate status to the caller only via
# a single exit code.  That's limiting.
#
# So, this provides an optional mechanism to provide more communicaton
# between parent and child script.  It is implemented as simply a
# file that is created by the child and source'ed by the parent.
#
# Children:  Don't roll-your-own file, instead call talkToCaller() to
# create it for you.  This will help assure consistency.
# =====================================================================
DHS_TTC_communicatonFile="dataToCaller.shinc"
DHS_TTC_systemReboot=false;     # assume that we don't need system restart

# =====================================================================
# $1 - true/false - T = indicate that something happened so that entire
#                   system must be restarted.
#                   F = ignore...have nothing to say about system restart
# =====================================================================
talkToCaller()
{
    if $1; then
        echo "DHS_TTC_systemReboot=$1" > "$DHS_TTC_communicatonFile"
    fi
}
getOptionalDataFromCallee()
{
    if [ -f "$DHS_TTC_communicatonFile" ]; then
        source "$DHS_TTC_communicatonFile"
        rm -f "$DHS_TTC_communicatonFile"
    fi
}

# =====================================================================
# Pre-105 release code had an mcfu driver problem:  If FU was busy and
# a pkt timed-out, the message queue would be forever screwed-up.
# This installs a driver w/ fixes for this situation.
#
# $1 - true/false - indicates success.  If false, do *NOT* do FU bundle update
# =====================================================================
maybeReplaceMcfuDriver()
{
    eval $1=true

    # isolote the major version.  e.g. "2" in 2.00.2014-02-12.11:30
    local curVersion=$(modinfo -F "version" mcfulink | grep "\.")
    curVersion=${curVersion##* }
    curVersion=${curVersion%%.*}

    if [ "$curVersion" == "1" ] || [ "$curVersion" == "2" ] || [ -z "$curVersion" ]; then
        if [ ! -f ./mcfulink101.ko ]; then
            msg "WARNING: Need to replace mcfu device driver, but new driver missing from bundle."
            return
        fi

        msg "Replacing the mcfu device driver"

        local removed=false

        local i1
        for i1 in $(seq 0 4); do
            modprobe -rfw mcfulink &> /dev/null
            lsmod | grep "mcfulink "
            if [ $? -ne 0 ]; then
                removed=true
                break
            fi
            sleep 5
        done

        if ! $removed; then
            msg "WARNING: Failed to remove original mcfu device driver"
            return
        fi


        insmod ./mcfulink101.ko
        sleep 2

        lsmod | grep "mcfulink "
        if [ $? -eq 0 ]; then
            return
        else
            # crap, try to restart the old one, then
            modprobe mcfulink
            sleep 2

            lsmod | grep "mcfulink "
            if [ $? -eq 0 ]; then
                msg "WARNING: Failed to replace original mcfu device driver"
            else
                msg "ERROR: Failed to reinstall the mcfu device driver"
                eval $1=false
            fi
        fi
    fi
}
