#!/bin/sh

source /mnt/ramdisk/utils.sh
setupexports

actComp="/bundle"
if [ -f /usr/bin/utils.sh ]; then
    source /usr/bin/utils.sh
else
    apiOK() { :; }
    apiNA() { :; }
    apiActive() { :; }
    apiActiveEcho() { :; }
    apiProgress() { :; }
    apiError() { :; }
fi

getMCpost lastPostMC

displayloadstatus()
{
msg ""
msg "=========================================="
msg "Software Component Load Summary:"
msg ""
msg "MC Software: $(decodeloadstatus $LOAD_STATUS_MC)"
msg "SC Software: $(decodeloadstatus $LOAD_STATUS_SC)"
msg "EC Software: $(decodeloadstatus $LOAD_STATUS_EC)"
msg "CPLD Software: $(decodeloadstatus $LOAD_STATUS_CPLD)"
msg "=========================================="
msg ""
    return
}

# =====================================================================
# This embodies the restart logic
# $1: "cpld" || "sc" || "mc"
# $2: failures: true/false
# $3: message
# =====================================================================
restart()
{
    local option=$1
    local failures=$2
    local message=$3

    # message to say what's happening
    apiActiveEcho $actComp "$message"

    # message for legacy customer scripts
    $failures && echo "RETURN_CODE: 10" || echo "RETURN_CODE: 8"

    # message to tell FTP to disconnect
    disconnectFTP

    # preprestart.sh unmounts /cfg, so we need to copy over our last.codeload.progress file manually here
    cp /mnt/ramdisk/codeload.progress /mnt/cfg/last.codeload.progress
    preprestart.sh

    # now kill FTP
    pkill -SIGUSR1 pure-ftpd;    # end ftp session

    case $option in   
 
    "cpld")
        # This will perform an LSC_Refresh.  The SC will reset Moose/FU/MC
        # we are about to LSC_Refresh, so we need to tell SC

        waitFor "SC_POST_SHUTDOWN_CODELOAD_CPLD_WDT_PROTECTED || SC_POST_BOOTING_PAUSED_WDT_PROTECTED" 1 "CPLD Reset Ready" result
        if $result; then
            # we have a WDT protected SC, so we can send the notification
            echo "Performing CPLD-RESTART notification"
            sleep 1
            postMC "MC_POST_PENDING_CPLD_RESET"
            sleep 1  # allow SC to see it
        else
            waitFor "SC_POST_SHUTDOWN_CODELOAD_CPLD || SC_POST_BOOTING_PAUSED" 1 "CPLD Reset Ready" result
            if ! $result; then
                echo "Unable to reset CPLD, SC not in correct state."
                apiError $actComp 1405 "Failure to reset CPLD, invalid SC state."
                exit $LOAD_STATUS_FAILED
            fi
        fi

        if [ -f /mnt/ramdisk/mc/i2c-codeload ]; then
            /mnt/ramdisk/mc/i2c-codeload --target cpld --reboot
        else
            i2c-codeload --target cpld --reboot
        fi
        ;;

    "sc")
        # This will restart the SC.  The SC will reset Moose/FU/MC
        postMC "$lastPostMC"
        sleep 1
        /mnt/ramdisk/capi.app --rebootsc
        ;;

    "mc")
        # This will restart the MC.
        postMC "$lastPostMC"
        rebootMc 15
        sleep 30;
        # reboot MC doesn't seem to be taking priority over the sleep, so it takes forever (literally forever) to actually reboot after a codeload. Dropping the time to 30.
        ;;

    *)
        echo "bad option"

    esac
}

# =====================================================================
# No matter what, run this function on exit
#
# Gallium HW has a problem.  When the SC reboots, it will also reboot
# the MC.  There is no fix for this.  In addition, when the cpld
# reboots, it takes down SC and MC similar to an SC reboot.
#
# So, I am using only two types of reboots on a Gallium:
#     1. MC, only
#     2. Everything restarts
# =====================================================================
exitTrapFcn()
{
    local exitCode=$?

    local ecAttempted
    local mcAttempted
    local scAttempted
    local cpldAttempted
    local cpldReg="/proc/moose/cpld/post_sc"

    if [ "$exitCode" -eq $LOAD_STATUS_NOT_ATTEMPTED ]; then
        markTasksAsNotAttempted
    fi
    wasAttempted $LOAD_STATUS_EC    ecAttempted
    wasAttempted $LOAD_STATUS_MC    mcAttempted
    wasAttempted $LOAD_STATUS_SC    scAttempted
    wasAttempted $LOAD_STATUS_CPLD  cpldAttempted

    displayloadstatus

    if [ "$exitCode" -eq "$LOAD_STATUS_FAILED" ]; then
        oneOrMoreFails=true
    else
        getOverallStat oneOrMoreFails
    fi

    # tell docodeload to self-destruct if something goes wrong
    touch do.reboot

    if $oneOrMoreFails; then
        echo "STATUS: *** Code Load Fail. ***"
        apiError $actComp 1050 "Some number of subcomponents failed."
    else
        echo "STATUS: Code load completed successfully."
        apiOK $actComp
    fi

    if $cpldAttempted; then
        # we have loaded, or attempted to load a CPLD (and possibly failed), so we need to ensure the SC
        # is in the right spot for CPLD load.
        # we have the PRE bootup and the POST bootup styles, so just wait for either of them

        echo "CPLD attempted"
        waitFor "SC_OS_SHUTDOWN_CODELOAD_CPLD || SC_OS_BOOTING_PAUSED" 1 "CPLD Reset Ready" result
        if $result; then
            restart "cpld" $oneOrMoreFails "Restarting CPLD. This will take a couple of minutes."
        else
            # we never got to the right state, recover
            # as a precautionary measure, if perhaps SC is codeloading, wait
            waitFor "! SC_OS_SHUTDOWN_CODELOAD" $((5*60)) "SC to complete codeload" result
            if $result; then
                restart "sc" $oneOrMoreFails "Restarting Storage Controller. This will take a couple of minutes."
            else
                apiActiveEcho $actComp "SC is not in the correct state for CPLD reset or restart.  Please correct this condition."
            fi
        fi
    elif $scAttempted; then
        # the SC auto reboots on an SC load, so it should be stopped at 0x74
        # we never got to the right state, recover

        echo "SC attempted"

        restart "sc" $oneOrMoreFails "Restarting Storage Controller. This will take a couple of minutes."

    elif $DHS_TTC_systemReboot; then
        # spi flash changes that require an SC reboot
        # just in case

        echo "DHS_TTC_systemReboot attempted"

        waitFor "SC_OS_ACTIVE || SC_OS_ACTIVEACTIVE || SC_OS_FAILOVER ||SC_OS_SHUTDOWN_NORMAL" 1 "SC Active" result
        if ! $result; then
            waitFor "! SC_OS_SHUTDOWN_CODELOAD" $((30*60)) "SC to complete codeload" result
            if ! $result; then
                apiActiveEcho $actComp "SC is still performing codeload, unable to restart."
            fi
        fi
        waitFor "SC_OS_ACTIVE || SC_OS_ACTIVEACTIVE || SC_OS_FAILOVER ||SC_OS_SHUTDOWN_NORMAL" 1 "SC Active" result
        if $result; then
            restart "sc" $oneOrMoreFails "Restarting Storage Controller. This will take a couple of minutes."
        else
            waitFor "SC_OS_SHUTDOWN_CODELOAD" 1 "Safe to Restart" result
            if $result; then
                apiActiveEcho $actComp "SC is still performing codeload, unable to restart."
            else
                apiActiveEcho $actComp "SC is not in correct state, unable to restart."
            fi
        fi
    else
        echo "MC reboot backstop"

        restart "mc" $oneOrMoreFails "Restarting Management Controller. This will take a couple of minutes."
    fi

    # unlock the post register no matter why the script terminated
    postMC "$lastPostMC"
    disconnectFTP
    checkforprogressmonitor; # end ftp session
}
trap exitTrapFcn SIGINT SIGTERM EXIT


# == CORE LOGIC =======================================================

apiActive $actComp "Loading bundle."

checkVersion

if [ -f /mnt/ramdisk/mc/utils.sh ]; then
    source /mnt/ramdisk/mc/utils.sh
fi

if [ -d mc ]; then
    checkintegrity mc
fi
markTasksAsPending

# ensure our capi.app has +x
if [ -f /mnt/ramdisk/capi.app ]; then
    if [ ! -x /mnt/ramdisk/capi.app ]; then
        chmod 755 /mnt/ramdisk/capi.app
    fi
else
# default to the capi.app in /usr/bin
    cp /usr/bin/capi.app /mnt/ramdisk
fi

#check to see if dopfu is already running
PFURUNNING=$(ps | grep "dopfu" | wc -l)
if [ $PFURUNNING -gt 3 ]
then
    echo "Partner Firmware Update in progress. Stopping code load to let the Partner Update finish."
    echo "Restarting Management Controller..."
    apiError $actComp 1051 "PFU in progress."
    exit $LOAD_STATUS_PENDING
fi

#hopefully this is no longer needed
#checkforreboot

#originated when appsv was loading SC and EC, and we needed to ensure we had space for logging. Or something. This is no longer the case, so yank.
#checkcfgfs

# are we healthy enough to continue with this codeload?. run this before stopping appsv.
echo "Checking system health."
healthcheck
if [ $? != 0 ]
then
    #healthcheck failed
    apiError $actComp 1053 "System health check failed."
    exit $LOAD_STATUS_NOT_ATTEMPTED
fi

# do we have unwritable cache data.  This is in the previous healthcheck for GLM200+, so only check if older versions
unwritableCacheCheck
if [ $? != 0 ]
then
    #unwritable cache failed
    apiError $actComp 1060 "Unwritable Cache check failed."
    exit $LOAD_STATUS_NOT_ATTEMPTED
fi

# burn to active :(
if [ ! -d /cfg/burn2active ]; then
    mkdir /cfg/burn2active
fi

#MC apps not needed and must be able to open CAPI pipe in capi.app so kill the apps
apiActiveEcho $actComp "Stopping Management Controller applications."
echo "`date`: Stopping Management Controller applications." >> $CPLDLOG
stopApps

#need to start the mini httpd server if it's present
startMsgServer

if [ ! -f is.pfu ]; then # only do this is if's a direct load, not PFU
    if [ "$LOAD_SC" == "true" ] || [ "$LOAD_EC" == "true" ]; then
        #make sure we can talk to the SC through /dev/mouse0 now
        ./capi.app --ping
        connectStatus=$?
        if [ $connectStatus -eq 0 ]; then
            apiActiveEcho $actComp "Initial connection to SC successful."
        else
            echo "STATUS: Code Load Failure"
            echo "*****************************************************************"
            echo "     The application is not able to communicate with the"
            echo "   storage controller. Verify the storage controller on this"
            echo "     FRU is operating correctly, then retry your request."
            echo "*****************************************************************"
            apiError $actComp 1054 "Failed to communicate with SC."
            exit $LOAD_STATUS_NOT_ATTEMPTED
        fi
    fi
fi

# We are now going to be defaulting the CPLD load style to "PRE" style
# for SC version GLS105 or greater, since they have WDT protection.  Anything less than
# that will require a POST load.
# Note, this code needs to happen here, because we've shut down the MC apps, so we can use
# the capi pipe (on pre GL105 versions).
SC_PRE_VERSION_MIN="GLS105"
getSCVersion scVersion
local scVersionPrefix=${scVersion:0:6}
echo "SCVersion=$scVersion"
if [[ "${scVersionPrefix}" \< "${SC_PRE_VERSION_MIN}" ]]
then
    touch /mnt/ramdisk/do.postCPLDstyle
else
    touch /mnt/ramdisk/do.preCPLDstyle
fi


# TODO: Arch problem:  We're not granular when writing version info.
#
# If one or more flash requests fail, then our version info is wrong.
# We're assume it will work and write all component bundle version
# info as one big ball rather than updating only if the flash succeeded.
if [ $USE_SYSTEM_VERSION == "true" ]; then
    if [ -f /cfg/version.txt ]
    then
        mv /cfg/version.txt /cfg/version.txt.prv
        cp system/version.txt /cfg/version.txt
    else
        cp system/version.txt /cfg/version.txt
    fi
fi

# ---------------------------------------------------------------------- #
# Specific component update calls
# ---------------------------------------------------------------------- #

# All componenets are responsible for their own flashing logic.
# Assumptions:
#   - No component flash operation will cause a reboot: it may be flagged
#     for reboot, however.
#   - A component's flash is not coupled / dependent on another component
#   - Each flash operation will return std status via an exit code

#
# CPLD load style (pre or post boot) is controlled by a file
#
if [ -f /mnt/ramdisk/do.preCPLDstyle ]; then
    echo "Using PRE CPLD load - shutdown CPLD"
    export postCPLDStyle=false
    export preCPLDStyle=true
else
    echo "Using POST CPLD load - booting paused"
    export postCPLDStyle=true
    export preCPLDStyle=false
fi

# Gallium - Rebooting the SC causes the MC to reboot
# notify the SC that we are starting codeload
postMC "MC_POST_CODELOAD"

componentFlashOrder="ec mc cfg sc cpld"

for targ in $componentFlashOrder; do
    upperName=$(echo $targ | tr '[:lower:]' '[:upper:]')
    eval exists=\$LOAD_$upperName
    if [ "$exists" == "true" ]; then
        cd $targ
            apiActive $actComp "Loading $upperName code."
            ./update.sh
            eval export LOAD_STATUS_$upperName=$?
            getOptionalDataFromCallee
        cd ..
    else
        apiNA "$actComp/$targ" "Not present in bundle"
    fi

    # protect against SC errors.  We ask SC to not reboot itself, but
    # it may anyway if it got an error
    if [ "$LOAD_STATUS_SC" -eq "$LOAD_STATUS_FAILED" ]; then
        echo "STATUS: Skipping other updates due to SC error."
        apiActive $actComp "Skipping other updates due to SC error."
        break
    fi
done

# finally, run exitTrapFcn() on any kind of exit
exit 0
