#!/bin/sh

# =====================================================================
# Pre-Gallium bundles were expected to reject themselves.  Gallium differs.
# Reject-a-bundle logic is installed on the Gallium and it uses bundleinfo
# properties to determine if a bundle is valid.
# =====================================================================
if [ -f /mnt/ramdisk/mc/fcns.shinc ]; then
    source /mnt/ramdisk/mc/fcns.shinc
else
    # use system-installed if newer not available
    source "fcns.shinc"
fi

event ()
{
    if [ -f /mnt/ramdisk/is.pfu ]
    then
        export EVENTCODE="PARTNER_FIRMWARE_UPDATE_IN_PROGRESS"
    else
        export EVENTCODE="FIRMWARE_UPDATE_IN_PROGRESS"
    fi

    /mnt/ramdisk/capi.app -x --postEvent $EVENTCODE.$1.$2.$3.$4

}

checkVersion()
{
    # Only check if this is a built image, not a development image
    [ ! -f /mnt/ramdisk/system/version.txt ] && return

    local TARGETED_BUNDLE_VERSION=$(inicntrl /mnt/ramdisk/system/version.txt system_bundle version)
    local BOARD_TYPE=$(/usr/bin/cpldtype)
    if [ "$BOARD_TYPE" != "348" ]; then
        apiError $actComp 1004 "Incompatible code bundle for this platform."
        echo "STATUS: Code Load Failure"
        echo "*****************************************************************"
        echo " Incompatible bundle $TARGETED_BUNDLE_VERSION for this platform"
        echo "        The management controller will restart."
        echo "        Please wait 2 minutes then retry your request."
        echo ""
        echo "        This management controller will now reboot."
        echo "*****************************************************************"
        disconnectFTP
        rebootMc 10
        sleep 30
        sync
        exit
    fi

    # ...if get here, then must be a Gallium.
    # Depend on docodeload+bundleinfo to protect us
}

setupexports()
{
    vmestr="_vme_gz"

    export LOAD_STATUS_NOT_ATTEMPTED=0
    export LOAD_STATUS_PENDING=1
    export LOAD_STATUS_SOURCE_FAILED=2; # exit code when 'source' fails
    export LOAD_STATUS_SUCCESS=3
    export LOAD_STATUS_FAILED=4

    export CFGFS_FULL_THRESHOLD=90

    export LOAD_MC="false"
    export LOAD_SC="false"
    export LOAD_EC="false"
    export LOAD_CPLD="false"
    export LOAD_CFG="false"

    export LOAD_STATUS_MC=$LOAD_STATUS_NOT_ATTEMPTED
    export LOAD_STATUS_EC=$LOAD_STATUS_NOT_ATTEMPTED
    export LOAD_STATUS_SC=$LOAD_STATUS_NOT_ATTEMPTED
    export LOAD_STATUS_CPLD=$LOAD_STATUS_NOT_ATTEMPTED
}

markTasksAsPending()
{
    if [ -d /mnt/ramdisk/cpld ]
    then
        export LOAD_CPLD="true"
        export LOAD_STATUS_CPLD="$LOAD_STATUS_PENDING"
    fi

    if [ -d /mnt/ramdisk/ec ]
    then
        export LOAD_EC="true"
        export LOAD_STATUS_EC="$LOAD_STATUS_PENDING"
    fi

    if [ -d /mnt/ramdisk/sc ]
    then
        export LOAD_SC="true"
        export LOAD_STATUS_SC="$LOAD_STATUS_PENDING"
    fi

    if [ -d /mnt/ramdisk/mc ]
    then
        export LOAD_MC="true"
        export LOAD_STATUS_MC="$LOAD_STATUS_PENDING"
    fi

    if [ -d /mnt/ramdisk/system ]
    then
        export USE_SYSTEM_VERSION="true"
    else
        export USE_SYSTEM_VERSION="false"
    fi

    if [ -d /mnt/ramdisk/cfg ]
    then
        export LOAD_CFG="true"
    fi


    return
}

markTasksAsNotAttempted()
{
    if [ $LOAD_STATUS_MC -le $LOAD_STATUS_PENDING ]; then
        export LOAD_STATUS_MC=$LOAD_STATUS_NOT_ATTEMPTED
    fi
    if [ $LOAD_STATUS_EC -le $LOAD_STATUS_PENDING ]; then
        export LOAD_STATUS_EC=$LOAD_STATUS_NOT_ATTEMPTED
    fi
    if [ $LOAD_STATUS_SC -le $LOAD_STATUS_PENDING ]; then
        export LOAD_STATUS_SC=$LOAD_STATUS_NOT_ATTEMPTED
    fi
    if [ $LOAD_STATUS_CPLD -le $LOAD_STATUS_PENDING ]; then
        export LOAD_STATUS_CPLD=$LOAD_STATUS_NOT_ATTEMPTED
    fi
}

# =====================================================================
# $1 - The return status to check
# $2 - returns:  variable name to place true/false
# =====================================================================
wasAttempted()
{
    local didAttempt=false
    if [ "$1" -gt "$LOAD_STATUS_NOT_ATTEMPTED" ]; then
        didAttempt=true
    fi

    eval $2=$didAttempt
}

decodeloadstatus()
{
    if [ "$1" -eq "$LOAD_STATUS_NOT_ATTEMPTED" ]; then
        echo "NOT ATTEMPTED"
        return
    fi
    if [ "$1" -eq "$LOAD_STATUS_PENDING" ]; then
        echo "PENDING"
        return
    fi
    if [ "$1" -eq "$LOAD_STATUS_SOURCE_FAILED" ]; then
        echo "FAILED"
        return
    fi
    if [ "$1" -eq "$LOAD_STATUS_SUCCESS" ]; then
        echo "SUCCESSFUL"
        return
    fi
    if [ "$1" -eq "$LOAD_STATUS_FAILED" ]; then
        echo "FAILED"
        return
    fi

    echo "Unknown"
    return
}

# =====================================================================
# $1 - returns:  variable name to place true/false
# =====================================================================
getOverallStat()
{
    local atLeastOneFail=false
    local stat

    for stat in \
        $LOAD_STATUS_MC \
        $LOAD_STATUS_EC \
        $LOAD_STATUS_SC \
        $LOAD_STATUS_CPLD; do
        if [ $stat -eq $LOAD_STATUS_SOURCE_FAILED ] \
           || [ $stat -eq $LOAD_STATUS_FAILED ]; then
           atLeastOneFail=true
        fi
    done

    eval $1=$atLeastOneFail
}

rebootMc()
{
    # just in case
    disconnectFTP
    # use clean reboot if available
    if [ -f /usr/bin/reboot.sh ]; then
        /usr/bin/reboot.sh $1
    elif [ -z $1 ]; then # no delay param passed
        reboot
    else # delayed reboot
        sleep $1
        reboot
    fi
}

checkforreboot()
{
    uptime=`awk '{ print $1; }' /proc/uptime`
    uptime=${uptime%%.*}
    if [ "$uptime" -gt "3600" ] && [ ! -f /mnt/ramdisk/is.pfu ]
    then
        echo ""
        echo "STATUS: Restarting Management Controller in 15 seconds."
        echo ""
        echo "This management controller will now automatically restart to allow you to load a new software bundle."
        echo "This automatic restart of this management controller will not effect data operations in any way."
        echo ""
        echo "************************************************************"
        echo "   You must initiate another software update within the"
        echo "         next 60 minutes to perform the update."
        echo "************************************************************"
        echo ""
        echo "Restarting management controller in 15 seconds..."
        echo "RETURN_CODE: 10"

        sync
        rebootMc 15
        sleep 10
    else
        echo "System has been up for $uptime seconds"
    fi
}


checkcfgfs()
{
    fssize=`df | grep /mnt/cfg | awk '{ printf("%d",$5); }'`
    if [ "$fssize" -gt "$CFGFS_FULL_THRESHOLD" ]
    then
        rm /cfg/log/*
        rm /cfg/log/active/*
        echo "STATUS: Code Load Failure"
        echo "*****************************************************************"
        echo " Insufficient resources are available to complete the codeload"
        echo "      operation.  The management controller will restart."
        echo "         Please wait 2 minutes then retry your request."
        echo ""
        echo "        This management controller will now reboot."
        echo "*****************************************************************"
        apiError $actComp 1057 "Insufficient resources to load code."
        disconnectFTP
        local __error=1
        sync
        rebootMc 15
        sleep 10
    else
        local __error=0
    fi
    eval $1=$__error
    return
}

#checkforcapi replaced with ping option to capi.app.
checkforcapi()
{
    if [ -d /mnt/ramdisk/sc ] || [ -d /mnt/ramdisk/ec ]
    then
        i="0"
        while [ ! -f /mem/capitimesynced ]
        do
            i=`expr $i + 1`
            if [ "$i" -eq "4" ]
            then
                echo "STATUS: Code Load Failure"
                echo "*****************************************************************"
                echo "     The application is not able to communicate with the"
                echo "   storage controller. Verify the storage controller on this"
                echo "     FRU is operating correctly, then retry your request."
                echo ""
                echo "        This management controller will now reboot."
                echo "*****************************************************************"
                sync
                rebootMc 15
                sleep 10
            fi

            echo "Pausing code load until base application becomes available..."
            sleep 20;
        done
    fi

    return
}

checkintegrity()
{
    if [ -f /cfg/no.integrity.check ]; then
        apiActiveEcho $actComp "$1 component integrity checking skipped!"
        msg "Integrity checking skipped!"
        return
    fi

    apiActiveEcho $actComp "Checking $1 component integrity."
    local md5stdout
    md5stdout=$(md5sum -c allfiles.md5)
    local result=$?

    if [ $result -ne 0 ]; then
        echo "ERROR: failed file integrity check."
        echo -e "$md5stdout\n"
        echo "Please retry your code load request."
        apiError $actComp 1055 "Component integrity failure."
        exit $LOAD_STATUS_FAILED
    fi
    apiActiveEcho $actComp "$1 component integrity checks passed."
}

# pre-230 did not clean up properly leaving child processes
# alive and holding docodeload open
checkforprogressmonitor()
{
    pm=$(ps | grep "progressmonitor" | wc -l)
    if [ "$pm" -gt "0" ]; then
        killall -9 progressmonitor
    fi
}

#code load may not proceed if 'show system' reports health as Bad. This
#function checks the health state and returns 0/1 for pass/fail.
healthcheck()
{
    local xmlResult=0
    local rtnValue=0
    #skip check if the following file exists
    if [ -e /cfg/codeload.ignore.health ]
    then
        echo "Skipping health check: /cfg/codeload.ignore.health file is present.";
        return 0
    fi

    if [ -e /mnt/ramdisk/do.force ]
    then
        echo "Skipping health check: Codeload being forced.";
        return 0
    fi

    #
    # Using the new cli command that generates xml output, check the health for updating any code.
    #
    xmlCommand "check firmware-upgrade-health"  "firmware-upgrade-health.xml" xmlResult
    if $xmlResult; then
        local healthCode=$(xmlcntrl "/mnt/ramdisk/firmware-upgrade-health.xml" "code-load-readiness" "overall-health")

        echo "Codeload health check complete. Health state: $healthCode"

        case "$healthCode" in

        "Green" ) ;;
        "Pass" ) ;;

        *)
            echo "STATUS: Firmware Update Can Not Proceed"
            echo "************************************************************************************"
            echo "           ERROR: System health insufficient to support firmware update."
            echo ""
            echo "    Reason: $healthCode"
            echo ""
            grep "readiness-reason" "/mnt/ramdisk/firmware-upgrade-health.xml" | cut -d \> -f2|cut -d \< -f1
            echo ""
            echo "Action: You must resolve the this condition before the firmware update will proceed."
            echo "************************************************************************************"
            event ERROR 1 MC_HEALTH_CHECK 1
            rtnValue=1
            ;;
        esac
    fi
    rm -f /mnt/ramdisk/firmware-upgrade-health.xml
    return $rtnValue
}

# =====================================================================
# get the MC version from the version.txt file
# $1 - returned result = true/false
# $2 - obtained result successfully = true/false
# =====================================================================
getMCVersion()
{
    eval $1=""; # mcVersion

    if [ -f /cfg/version.txt ]
    then
        eval $1=$(inicntrl /cfg/version.txt MC_system version);
    fi
}

# =====================================================================
# get the SC version from the SC directly
# note: this uses the same capi pipe as appsv, so it needs to be shut down
# $1 - returned version
# =====================================================================
getSCVersion()
{
    eval $1=$(/mnt/ramdisk/capi.app --scinfo | cut -d '|' -f 4)
}

# =====================================================================
# see if the running version has unwritable cache data
# $1 - returned result = true/false
# $2 - obtained result successfully = true/false
# =====================================================================
hasUnwritableCache()
{
    eval $1=false
    eval $2=false
    for i1 in $(seq 0 5); do
        local xmlResult=0
        xmlCommand "show unwritable-cache" "unwritablecache.xml" xmlResult

        if $xmlResult; then
            # 0 means there was unwritable cache, 1 means there's none
            local nouca=$(grep "unwritable-a-percentage" /mnt/ramdisk/unwritablecache.xml | grep ">0<" | wc -l)
            local noucb=$(grep "unwritable-b-percentage" /mnt/ramdisk/unwritablecache.xml | grep ">0<" | wc -l)
            rm /mnt/ramdisk/unwritablecache.xml
            if [[ $nouca -eq 0 -o $noucb -eq 0 ]]
            then
                eval $1=true
            else
                eval $1=false
            fi
            eval $2=true
            return 0
        else
            eval $2=false
        fi
    done
}

#code load may not proceed if 'show unwritable-cache' reports unwritable data on A or B.
unwritableCacheCheck()
{
    #skip check if the following file exists
    if [ -e /cfg/codeload.ignore.health ]
    then
        echo "Skipping unwritable cache check: /cfg/codeload.ignore.health file is present.";
        return 0
    fi

    if [ -e /mnt/ramdisk/do.force ]
    then
        echo "Skipping unwritable cache check: Codeload being forced.";
        return 0
    fi

    # this is the version where we introduced the healthcheck.  For prior versions, we use a scripted check for unwritable cache data present
    doCheck=false
    MC_HC_VERSION_MIN="GLM200"
    getMCVersion mcVersion
    local mcVersionPrefix=${mcVersion:0:6}
    echo "MCVersion=$mcVersion"
    if [ "${mcVersionPrefix}" \< "${MC_HC_VERSION_MIN}" ]
    then
        doCheck=true
    fi

    if $doCheck; then
        echo "Check for Unwritable Cache Data"
        hasUnwritableCache hasUC success
        if $success; then
            if $hasUC; then
                echo "STATUS: Firmware Update Can Not Proceed"
                echo "************************************************************************************"
                echo "                ERROR: System has Unwritable Cache Data present."
                echo ""
                echo "Action: You must resolve the this condition before the firmware update will proceed."
                echo "************************************************************************************"
                event ERROR 1 UNWRITABLE_CACHE 1
                return 1
            else
                echo "No Unwritable Cache Data present."
                return 0
            fi
        else
            echo "STATUS: Firmware Update Can Not Proceed"
            echo "***********************************************************************************************"
            echo "           ERROR: Unable to determine if Unwritable Cache Data is present."
            echo ""
            echo "Action: Please ensure your storage controller is available before performing a firmware update."
            echo "***********************************************************************************************"
            event ERROR 1 UNWRITABLE_CACHE 2
            return 1
        fi
    fi
    return 0
}


# check to see if appsv is dow, if so we need to reboot regardless of what we flashed
checkforappdown()
{
    appsvup=$(ps | grep appsv | grep -v grep | wc -l)
    if [[ $appsvup -gt 0 ]]
    then
        eval $1=false
    else
        eval $1=true
    fi
}

# waitFor SC states
# $1 - "waitFor expression string" - expression string indicating the states and condition logic
# $2 - timeout (in seconds) for waitFor to stop if it hasn't met the condition
# $3 - "message to print" - the message to print out every 15 seconds
# $4 - return the success/error (true if success|false if timeout)
# $5 - "capi" - if present, do Capi request to refine states
waitFor()
{
    local wfs="[debug|timeout:$2|message:$3|freq:15|noCapi]$1"
    if [ "$5" == "capi" ]; then
        wfs="[debug|timeout:$2|message:$3|freq:15]$1"
    fi
    /mnt/ramdisk/capi.app --waitFor "$wfs" --component $actComp
    if [ $? -eq 0 ]; then
        eval $4=true
    else
        eval $4=false
    fi
}

# postMC
# $1 - MC post value or symbol
postMC()
{
    /mnt/ramdisk/capi.app --postMC $1
}

# getMCpost
# $1 = variable to return value
getMCpost()
{
    eval $1=$(/mnt/ramdisk/capi.app --getMCpost)
}


